## Dense Contrastive Features for Correlation Filters
By Jaime Spencer Martin, Richard Bowden & Simon Hadfield

{jaime.spencer, r.bowden, s.hadfield} @surrey.ac.uk

### Introduction
Dense Contrastive Features for Correlation Filters (DCFCF) extends on
previous work based on correlation filters applied to feature
representations of the tracked object. Previously, this consisted of
carefully engineered hand-crafted features, such as SIFT or HOG.
More recently, with the rise of deep learning and the availability
of larger datasets, many methods have opted for machine learnt
descriptors. Methods such as CFWCR use the activations from intermediate
layers in an VGG network pretrained on the ImageNet dataset.

These pretrained features, however, carry some inherent limitations.
Most of these networks focus on the classification of a specific set of
classes, requiring us to make assumptions about how they generalize to
new categories. We introduce a new type of dense feature descriptors,
trained specifically for the comparison of unknown objects.

Our features are generated from a Fully Convolutional Network, trained
on pairs of images from a subset of the Kitti odometry dataset. Pairs
are preprocessed to obtain corresponding matching points (~15000 matches
per pair), leading to large amounts of training data from less images.
The network is trained to optimize ''pixel-wise" contrastive loss, where
similar pairs (matches) are encouraged to be represented closely in the
feature space and dissimilar pairs (non-matches) are separated by at
least a set margin (0.5 in our tests). The network can be trained to
produce different dimensionality output by simply modifying the final layer.

These generic comparison features lead to a more robust representation
of a priori unknown objects, largely increasing the resolution compared
to intermediate layers, whilst maintaining a reasonable dimensionality.
This results is a slight increase in performance, along with a higher
resistance to occlusions or missing targets.

## Requirements
1. OS Requirements

```
- Tested on Ubuntu >= 14.04
- Matlab >= R2015a
- Python 2.7
     Tensorflow 1.4
     Numpy
     PIL
```
2. Download the VGG pretrained model

```
cd ./feature_extraction/networks
wget http://www.vlfeat.org/matconvnet/models/imagenet-vgg-m-2048.mat. 
mv  imagenet-vgg-m-2048.mat imagenet-vgg-m-2048-cut.mat
```

3. Download the Siamese pretrained model

```
cd ./feature_tf/models
wget http://personal.ee.surrey.ac.uk/Personal/S.Hadfield/jaime/models.zip
```

3. Download the Python Tensorflow environment
```
cd ./pyenv_cpu
wget http://personal.ee.surrey.ac.uk/Personal/S.Hadfield/jaime/pyenv_cpu.zip
```

4. If there are issues setting the Python executable in Matlab
```
pyversion(path/to/repo/pyenv_cpu/bin/python)
```

5. If there are issues with invalid MEX files try adding the following
~/.bashrc or equivalent
```
export LD_PRELOAD=/usr/lib/x86_64-linux-gnu/libstdc++.so.6
```

## Demo
1、 run demo_DCFCF

## VOT 
2、[VOT Intergration] ./vot2018_trax
