% ***********************************************
% Comput image feature from Kitti trained network
% ***********************************************
function out_feat = get_image_feat_tf(im, dparams)

% Number of output features
out_feat = cell(1, 1, length(dparams.tf_layers));

% Compute feature for each scale.
% Matlab & Python conversions make this more annoying than it should be...
for j = 1:size(im, 4)
    % Select scale
    I = im(:,:,:,j);
    
    % Flatten into numpy array
    py_im = py.numpy.array(I(:).', py.numpy.uint8);
    
    % Get network prediction
    out = dparams.infer_module.run(dparams.sess, dparams.net, py_im, dparams.tf_layers);

    % Generate full output feature for each layer
    for i = 1:length(out)
        arr = out{i};
        
        % Convert numpy array into Matlab array
        data = double(py.array.array('d',py.numpy.nditer(arr)));   
        
        % Reshape to correct shape
        shape = cell2mat(cell(arr.shape));
        new_feat = reshape(data, fliplr(shape));
        new_feat = permute(new_feat, [3 2 1]);
        
        % Concatenate with previous scales
        out_feat{i} = cat(4, out_feat{i}, new_feat);
    end
end

end

