
from __future__ import absolute_import, division, print_function
import tensorflow as tf


def conv2d(x, k_size, n_filters, strides, relu, name='conv'):

    with tf.name_scope(name):
        n_prev_filters = x.get_shape()[3]
        shape = [k_size, k_size, n_prev_filters, n_filters]
        strides = [1, strides, strides, 1]

        w_init = tf.contrib.layers.xavier_initializer()
        b_init = tf.constant(0, shape=[n_filters], dtype=tf.float32)

        with tf.variable_scope(name):
            w = tf.get_variable('W', dtype=tf.float32, shape=shape, initializer=w_init)
            b = tf.get_variable('b', dtype=tf.float32, initializer=b_init)

        conv = tf.nn.conv2d(x, w, strides=strides, padding='SAME') + b
        return tf.nn.relu(conv, name='relu') if relu else conv


def max_pool(x, name='pool'):
    with tf.name_scope(name):
        return tf.nn.max_pool(x, ksize=[1, 2, 2, 1], strides=[1, 3, 3, 1], padding='SAME')


def lrn(x, name='lrn'):  # Bias of 0 breaks it
    with tf.name_scope(name):
        return tf.nn.local_response_normalization(x, depth_radius=2, bias=1.0, alpha=0.0001, beta=0.5)


def dropout(x, keep_prob, name='dropout'):
    with tf.name_scope(name):
        return tf.nn.dropout(x, keep_prob=keep_prob)


def resize(x, shape, name='resize'):
    with tf.name_scope(name):
        return tf.image.resize_images(x, size=shape, method=tf.image.ResizeMethod.BILINEAR)


def upconv2d(x, shape, k_size, n_filters, strides, name='upconv'):
    with tf.name_scope(name):
        res = resize(x, shape)
        with tf.variable_scope(name):
            return conv2d(res, k_size, n_filters, strides, relu=False)


def main():
    pass


if __name__ == "__main__":
    main()
